# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long, too-many-statements, too-many-locals, too-many-branches

from azure.cli.core.azclierror import CLIError, AzureResponseError, ResourceNotFoundError
from azure.appconfiguration import ConfigurationSettingsFilter
from azure.cli.core.commands.progress import IndeterminateStandardOut
from azure.core.exceptions import HttpResponseError

from ._constants import ProvisioningStatus, StatusCodes, SearchFilterOptions, SnapshotFilterFields
from ._snapshotmodels import SnapshotQueryFields, Snapshot
from ._utils import get_appconfig_data_client


# Snapshot commands #

def create_snapshot(cmd,
                    snapshot_name,
                    filters,
                    name=None,
                    connection_string=None,
                    auth_mode='key',
                    endpoint=None,
                    retention_period=None,
                    composition_type=None,
                    tags=None):

    client = get_appconfig_data_client(cmd, name, connection_string, auth_mode, endpoint)

    configurationSettingsFilters = [ConfigurationSettingsFilter(
        key=x.get(SnapshotFilterFields.KEY),
        label=x.get(SnapshotFilterFields.LABEL),
        tags=x.get(SnapshotFilterFields.TAGS)) for x in filters]

    progress = IndeterminateStandardOut()
    progress.write({"message": "Starting"})

    try:
        config_snapshot_poller = client.begin_create_snapshot(
            snapshot_name,
            configurationSettingsFilters,
            composition_type=composition_type,
            retention_period=retention_period,
            tags=tags)

        # Poll snapshot creation status
        while config_snapshot_poller.status() != ProvisioningStatus.SUCCEEDED:
            progress.spinner.step(label="Running")
            config_snapshot_poller.wait(1)

        return Snapshot.from_configuration_snapshot(config_snapshot_poller.result())

    except HttpResponseError as exception:
        if exception.status_code == StatusCodes.CONFLICT:
            raise AzureResponseError("A snapshot with name '{}' already exists.".format(snapshot_name))

        raise AzureResponseError(str(exception))

    except Exception as exception:
        raise CLIError("Failed to create snapshot. {}".format(str(exception)))

    finally:
        progress.clear()


def show_snapshot(cmd,
                  snapshot_name,
                  name=None,
                  fields=None,
                  connection_string=None,
                  auth_mode='key',
                  endpoint=None):

    query_fields = []

    if fields:
        for field in fields:
            if field == SnapshotQueryFields.ALL:
                query_fields.clear()
                break
            query_fields.append(field.name.lower())

    client = get_appconfig_data_client(cmd, name, connection_string, auth_mode, endpoint)

    try:
        result = client.get_snapshot(snapshot_name, fields=query_fields)

    except HttpResponseError as exception:
        if exception.status_code == StatusCodes.NOT_FOUND:
            raise ResourceNotFoundError("No snapshot with name '{}' was found.".format(snapshot_name))

        raise AzureResponseError(str(exception))

    except Exception as exception:
        raise CLIError("Request failed. {}".format(str(exception)))

    snapshot = Snapshot.from_configuration_snapshot(result)

    if not query_fields:
        return snapshot

    partial_snapshot = {}

    for field in query_fields:
        partial_snapshot[field] = snapshot.__dict__[field]

    return partial_snapshot


def list_snapshots(cmd,
                   snapshot_name=SearchFilterOptions.ANY_KEY,
                   status=None,
                   fields=None,
                   name=None,
                   top=None,
                   all_=None,
                   connection_string=None,
                   auth_mode='key',
                   endpoint=None):

    query_fields = []

    if fields:
        for field in fields:
            if field == SnapshotQueryFields.ALL:
                query_fields.clear()
                break
            query_fields.append(field.name.lower())

    client = get_appconfig_data_client(cmd, name, connection_string, auth_mode, endpoint)

    try:
        snapshots_iterable = client.list_snapshots(name=snapshot_name, status=status, fields=query_fields)

    except HttpResponseError as exception:
        raise AzureResponseError(str(exception))

    except Exception as exception:
        raise CLIError('Request failed, {}'.format(str(exception)))

    if all_:
        top = float('inf')
    elif top is None:
        top = 100

    snapshots = []
    current_snapshot_count = 0

    for config_snapshot in snapshots_iterable:
        snapshot = Snapshot.from_configuration_snapshot(config_snapshot)
        if query_fields:
            partial_snapshot = {}
            for field in query_fields:
                partial_snapshot[field] = snapshot.__dict__[field]
            snapshots.append(partial_snapshot)
        else:
            snapshots.append(snapshot)

        current_snapshot_count += 1

        if current_snapshot_count >= top:
            break

    return snapshots


def archive_snapshot(cmd,
                     snapshot_name,
                     name=None,
                     connection_string=None,
                     auth_mode='key',
                     endpoint=None):

    client = get_appconfig_data_client(cmd, name, connection_string, auth_mode, endpoint)

    try:
        result = client.archive_snapshot(snapshot_name)

        return Snapshot.from_configuration_snapshot(result)

    except HttpResponseError as exception:
        if exception.status_code == StatusCodes.NOT_FOUND:
            raise ResourceNotFoundError("No snapshot with name '{}' was found.".format(snapshot_name))

        raise AzureResponseError(str(exception))

    except Exception as exception:
        raise CLIError("Failed to archive snapshot. {}".format(str(exception)))


def recover_snapshot(cmd,
                     snapshot_name,
                     name=None,
                     connection_string=None,
                     auth_mode='key',
                     endpoint=None):

    client = get_appconfig_data_client(cmd, name, connection_string, auth_mode, endpoint)

    try:
        result = client.recover_snapshot(snapshot_name)

        return Snapshot.from_configuration_snapshot(result)

    except HttpResponseError as exception:
        if exception.status_code == StatusCodes.NOT_FOUND:
            raise ResourceNotFoundError("No snapshot with name '{}' was found.".format(snapshot_name))

        raise AzureResponseError(str(exception))

    except Exception as exception:
        raise CLIError("Failed to recover snapshot. {}".format(str(exception)))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from time import sleep
import unittest
from knack.util import CLIError

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, JMESPathCheck


class TestMonitorActivityLogAlert(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_create_default_activity_log_alert(self, resource_group):
        name, scope, _ = self._create_and_test_default_alert(resource_group)

        self.kwargs['name'] = name

        # TODO: Re-enable more specific checking when #5155 is addressed.
        # with self.assertRaises(CLIError) as cm:
        #     self.cmd('az monitor activity-log alert create -n {} -g {}'.format(name, resource_group))
        # self.assertEqual('The activity log alert {} already exists in resource group {}.'.format(name, resource_group),
        #                  str(cm.exception))
        self.cmd('az monitor activity-log alert create -n {name} -g {rg}', expect_failure=True)

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_create_disabled_activity_log_alert(self, resource_group):
        name = self.create_random_name('clialert', 32)

        self.cmd('az monitor activity-log alert create -n {} -g {} --disable -ojson'.format(name, resource_group),
                 checks=[JMESPathCheck('name', name),
                         JMESPathCheck('enabled', False),
                         JMESPathCheck('location', 'Global'),
                         JMESPathCheck('length(scopes)', 1),
                         JMESPathCheck('length(condition.allOf)', 1),
                         JMESPathCheck('length(actions.actionGroups)', 0),
                         JMESPathCheck('condition.allOf[0].equals', 'ServiceHealth'),
                         JMESPathCheck('condition.allOf[0].field', 'category')])

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_create_update_activity_log_alert_anyof_conditon(self, resource_group):
        name = self.create_random_name('clialert', 32)
        create_cmd = 'az monitor activity-log alert create -n {} -g {} --disable'.format(name, resource_group)  + ' --all-of "[{{any-of:[{{field:level,equals:Informational}}]}}]"'
        self.cmd(create_cmd,
                 checks=[JMESPathCheck('name', name),
                         JMESPathCheck('enabled', False),
                         JMESPathCheck('location', 'Global'),
                         JMESPathCheck('length(scopes)', 1),
                         JMESPathCheck('length(condition.allOf)', 2),
                         JMESPathCheck('length(actions.actionGroups)', 0),
                         JMESPathCheck('condition.allOf[0].anyOf[0].field', 'level'),
                         JMESPathCheck('condition.allOf[0].anyOf[0].equals', 'Informational'),
                         JMESPathCheck('condition.allOf[1].field', 'category'),
                         JMESPathCheck('condition.allOf[1].equals', 'ServiceHealth'),
                         ])
        update_cmd = 'az monitor activity-log alert update -n {} -g {}'.format(name, resource_group) + ' --all-of "[{{any-of:[{{field:level,equals:Error}}]}}]"'
        self.cmd(update_cmd,
                 checks=[
                     JMESPathCheck('length(condition.allOf)', 2),
                     JMESPathCheck('condition.allOf[0].anyOf[0].field', 'level'),
                     JMESPathCheck('condition.allOf[0].anyOf[0].equals', 'Error'),
                     JMESPathCheck('condition.allOf[1].field', 'category'),
                     JMESPathCheck('condition.allOf[1].equals', 'ServiceHealth'),
                 ])

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_create_full_fledged_activity_log_alert(self, resource_group):
        scope = self.cmd('az group show -n {} -ojson'.format(resource_group)).get_output_in_json()['id']

        action_name = self.create_random_name('cliact', 32)
        action_rid = self.cmd('az monitor action-group create -n {} -g {} -ojson'.format(action_name, resource_group)) \
            .get_output_in_json()['id']

        alert_name = self.create_random_name('clialert', 32)
        self.cmd('az monitor activity-log alert create -n {} -g {} -s {} --action {} --condition {} and {} --disable '
                 '-ojson'.format(alert_name, resource_group, scope, action_name, 'category=Security', 'level=Error'),
                 checks=[JMESPathCheck('name', alert_name),
                         JMESPathCheck('enabled', False),
                         JMESPathCheck('location', 'Global'),
                         JMESPathCheck('length(scopes)', 1),
                         JMESPathCheck('length(condition.allOf)', 2),
                         JMESPathCheck('length(actions.actionGroups)', 1),
                         JMESPathCheck('scopes[0]', scope),
                         JMESPathCheck('actions.actionGroups[0].actionGroupId', action_rid, case_sensitive=False)])

        # test monitor activity-log alert list
        self.cmd('az monitor activity-log alert list -g {} -ojson'.format(resource_group),
                 checks=[JMESPathCheck('length(@)', 1)])

        # test monitor activity-log list-categories
        self.cmd('monitor activity-log list-categories',
                 checks=[self.check('length(@)', 8)])

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_activity_log_alert_update_action(self, resource_group):
        cmd_action = 'az monitor action-group create -n {} -g {} --query id -otsv'
        action_names = [self.create_random_name('cliact', 32) for _ in range(3)]
        action_rid = [self.cmd(cmd_action.format(name, resource_group)).output.strip() for name in action_names]
        name, scope, _ = self._create_and_test_default_alert(resource_group)

        # add one action group
        self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} -ojson'
                 .format(name, resource_group, action_rid[0]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 1),
                         JMESPathCheck('actions.actionGroups[0].actionGroupId', action_rid[0])])

        # add reset action group
        self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} --reset -ojson'
                 .format(name, resource_group, action_names[1]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 1),
                         JMESPathCheck('actions.actionGroups[0].actionGroupId', action_rid[1], case_sensitive=False)])
        # action-group generated id's subscriptionid is uppercase while local action_rid generated is lowercase...

        # add two action groups
        self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} {} -ojson'
                 .format(name, resource_group, action_rid[0], action_names[2]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 3)])

        # repeatedly add one action should have no effect
        self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} -ojson'
                 .format(name, resource_group, action_rid[0]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 3)])

        # list all actions
        state = self.cmd('az monitor activity-log alert show -n {} -g {} -ojson'
                         .format(name, resource_group)).get_output_in_json()

        for action_group in state['actions']['actionGroups']:
            self.assertEqual(action_group['webhookProperties'], {})

        # update one action's webhook properties
        state = self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} -w purpose=test -ojson'
                         .format(name, resource_group, action_names[0]),
                         checks=[JMESPathCheck('length(actions.actionGroups)', 3)]).get_output_in_json()

        for action_group in state['actions']['actionGroups']:
            if action_group['actionGroupId'].lower() == action_rid[0].lower():
                self.assertEqual(action_group['webhookProperties']['purpose'], 'test')
            else:
                self.assertEqual(action_group['webhookProperties'], {})

        # update webhook properties in strict mode render error
        with self.assertRaises(ValueError):
            self.cmd('az monitor activity-log alert action-group add -n {} -g {} -a {} -w purpose=error-trigger '
                     '--strict -ojson'.format(name, resource_group, action_rid[0]))

        # remove one action
        self.cmd('az monitor activity-log alert action-group remove -n {} -g {} -a {} -ojson'
                 .format(name, resource_group, action_names[0]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 2)])

        # remove missing action renders no effect
        self.cmd('az monitor activity-log alert action-group remove -n {} -g {} -a {} -ojson'
                 .format(name, resource_group, action_rid[0]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 2)])

        # remove two actions
        self.cmd('az monitor activity-log alert action-group remove -n {} -g {} -a {} {} -ojson'
                 .format(name, resource_group, action_names[1], action_rid[2]),
                 checks=[JMESPathCheck('length(actions.actionGroups)', 0)])

        # delete
        self.cmd('az monitor activity-log alert delete -n {} -g {}'.format(name, resource_group))

        # show
        # with self.assertRaises(CLIError):
        #     self.cmd('az monitor activity-log alert show -n {} -g {}'.format(name, resource_group))
        self.cmd('az monitor activity-log alert show -n {} -g {}'.format(name, resource_group), expect_failure=True)

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_activity_log_alert_update_condition(self, resource_group):
        name, scope, _ = self._create_and_test_default_alert(resource_group)
        update_cmd = 'az monitor activity-log alert update -n {} -g {} '.format(name, resource_group)

        state = self.cmd(update_cmd + '-c category=Security and level=Informational').get_output_in_json()
        condition_dict = {each['field']: each['equals'] for each in state['condition']['allOf']}
        self.assertEqual(2, len(condition_dict))
        self.assertEqual('Informational', condition_dict['level'])
        self.assertEqual('Security', condition_dict['category'])

        state = self.cmd(update_cmd + '-c level=Error and category=Security and resourceGroup={}'.format(
            resource_group)).get_output_in_json()
        condition_dict = {
            each['field']: each['equals']
            for each in state['condition']['allOf']
        }
        self.assertEqual(3, len(condition_dict))
        self.assertEqual('Error', condition_dict['level'])
        self.assertEqual('Security', condition_dict['category'])
        self.assertEqual(resource_group, condition_dict['resourceGroup'])

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_activity_log_alert_update_scope(self, resource_group):
        name, scope, _ = self._create_and_test_default_alert(resource_group)
        self.cmd('az monitor activity-log alert update -n {} -g {} -c level=Error and category=Security'.format(name, resource_group))

        resource_group_id = self.cmd("az group show -n {} --query id -otsv".format(resource_group)).output.strip()

        action_group_id = self.cmd('az monitor action-group create -n {} -g {} --query id -otsv'.format(name, resource_group)).output.strip()

        self.cmd('az monitor activity-log alert scope add -n {} -g {} -s {}'.format(name, resource_group, resource_group_id),
                 checks=[JMESPathCheck('length(scopes)', 2)])

        self.cmd('az monitor activity-log alert scope add -n {} -g {} -s {}'.format(name, resource_group, action_group_id),
                 checks=[JMESPathCheck('length(scopes)', 3)])

        self.cmd('az monitor activity-log alert scope add -n {} -g {} -s {} --reset'
                 .format(name, resource_group, action_group_id),
                 checks=[JMESPathCheck('length(scopes)', 1)])

        self.cmd('az monitor activity-log alert scope add -n {} -g {} -s {} {} {} --reset'
                 .format(name, resource_group, scope, resource_group_id, action_group_id),
                 checks=[JMESPathCheck('length(scopes)', 3)])

        self.cmd('az monitor activity-log alert scope remove -n {} -g {} -s {}'
                 .format(name, resource_group, scope),
                 checks=[JMESPathCheck('length(scopes)', 2)])

    def _create_and_test_default_alert(self, rg):
        name = self.create_random_name('clialert', 32)
        scope = self.cmd('az group show -n {} -ojson'.format(rg)).get_output_in_json()['id'].split("/resourceGroups")[0]
        self.cmd('az monitor activity-log alert create -n {} -g {} -ojson'.format(name, rg),
                 checks=[JMESPathCheck('name', name),
                         JMESPathCheck('enabled', True),
                         JMESPathCheck('location', 'Global'),
                         JMESPathCheck('length(scopes)', 1),
                         JMESPathCheck('length(condition.allOf)', 1),
                         JMESPathCheck('length(actions.actionGroups)', 0),
                         JMESPathCheck('scopes[0]', scope),  # default scope is this rg id, this changed, default is subid
                         JMESPathCheck('condition.allOf[0].equals', 'ServiceHealth'),
                         JMESPathCheck('condition.allOf[0].field', 'category')]).get_output_in_json()

        # ensure the alert is created, retry once
        retry = 2
        last_exception = Exception('Unknown exception. This is not expected to be thrown from the test.')
        for _ in range(retry):
            sleep(3 * retry)
            try:
                return name, scope, self.cmd('az monitor activity-log alert show -n {} -g {} -ojson'.format(name, rg))
            except CLIError as ex:
                last_exception = ex
                pass
        else:
            raise last_exception



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from collections import namedtuple
import os
import sys
import unittest
from unittest import mock
import tempfile
import json
import platform

from azure.cli.core.util import \
    (get_file_json, truncate_text, shell_safe_json_parse, b64_to_hex, hash_string, random_string,
     open_page_in_browser, can_launch_browser, handle_exception, ConfiguredDefaultSetter, send_raw_request,
     should_disable_connection_verify, parse_proxy_resource_id, get_az_user_agent, get_az_rest_user_agent,
    _get_parent_proc_name, is_wsl, run_cmd, run_az_cmd, roughly_parse_command, is_same_origin)
from azure.cli.core.mock import DummyCli


class TestUtils(unittest.TestCase):

    def _assert_headers(self, request_headers, expected_headers):
        actual_headers = dict(request_headers)

        encodings = {
            encoding.strip().lower()
            for encoding in actual_headers.get('Accept-Encoding', '').split(',')
            if encoding.strip()
        }
        self.assertIn('gzip', encodings)
        self.assertIn('deflate', encodings)

        actual_headers.pop('Accept-Encoding', None)
        self.assertDictEqual(actual_headers, expected_headers)

    def test_load_json_from_file(self):
        _, pathname = tempfile.mkstemp()

        # test good case
        with open(pathname, 'w') as good_file:
            good_file.write('{"key1":"value1", "key2":"value2"}')
        result = get_file_json(pathname)
        self.assertEqual('value2', result['key2'])

        # test error case
        with open(pathname, 'w') as bad_file:
            try:
                bad_file.write('{"key1":"value1" "key2":"value2"}')
                get_file_json(pathname)
                self.fail('expect throw on reading from badly formatted file')
            except Exception as ex:  # pylint: disable=broad-except
                self.assertTrue(str(ex).find(
                    'contains error: Expecting value: line 1 column 1 (char 0)'))

    def test_truncate_text(self):
        expected = 'stri [...]'
        actual = truncate_text('string to shorten', width=10)
        self.assertEqual(expected, actual)

    def test_truncate_text_not_needed(self):
        expected = 'string to shorten'
        actual = truncate_text('string to shorten', width=100)
        self.assertEqual(expected, actual)

    def test_truncate_text_zero_width(self):
        with self.assertRaises(ValueError):
            truncate_text('string to shorten', width=0)

    def test_truncate_text_negative_width(self):
        with self.assertRaises(ValueError):
            truncate_text('string to shorten', width=-1)

    def test_shell_safe_json_parse(self):
        dict_obj = {'a': 'b & c'}
        list_obj = [{'a': 'b & c'}]
        failed_strings = []

        valid_dict_strings = [
            '{"a": "b & c"}',
            "{'a': 'b & c'}",
            "{\"a\": \"b & c\"}"
        ]
        for string in valid_dict_strings:
            actual = shell_safe_json_parse(string)
            try:
                self.assertEqual(actual, dict_obj)
            except AssertionError:
                failed_strings.append(string)

        valid_list_strings = [
            '[{"a": "b & c"}]',
            "[{'a': 'b & c'}]",
            "[{\"a\": \"b & c\"}]"
        ]
        for string in valid_list_strings:
            actual = shell_safe_json_parse(string)
            try:
                self.assertEqual(actual, list_obj)
            except AssertionError:
                failed_strings.append(string)

        self.assertEqual(
            len(failed_strings), 0,
            'The following patterns failed: {}'.format(failed_strings))

    def test_hash_string(self):
        def _run_test(length, force_lower):
            import random
            # test a bunch of random strings for collisions
            test_values = []
            for x in range(100):
                rand_len = random.randint(50, 100)
                test_values.append(random_string(rand_len))

            # test each value against eachother to verify hashing properties
            equal_count = 0
            for val1 in test_values:
                result1 = hash_string(val1, length, force_lower)

                # test against the remaining values and against itself, but not those which have
                # come before...
                test_values2 = test_values[test_values.index(val1):]
                for val2 in test_values2:
                    result2 = hash_string(val2, length, force_lower)
                    if val1 == val2:
                        self.assertEqual(result1, result2)
                        equal_count += 1
                    else:
                        self.assertNotEqual(result1, result2)
            self.assertEqual(equal_count, len(test_values))

        # Test digest replication
        _run_test(100, False)

        # Test force_lower
        _run_test(16, True)

    def test_proxy_resource_parse(self):
        mock_proxy_resource_id = "/subscriptions/0000/resourceGroups/clirg/" \
                                 "providers/Microsoft.Network/privateEndpoints/cli/" \
                                 "privateLinkServiceConnections/cliPec/privateLinkServiceConnectionsSubTypes/cliPecSubName"
        result = parse_proxy_resource_id(mock_proxy_resource_id)
        valid_dict_values = {
            'subscription': '0000',
            'resource_group': 'clirg',
            'namespace': 'Microsoft.Network',
            'type': 'privateEndpoints',
            'name': 'cli',
            'child_type_1': 'privateLinkServiceConnections',
            'child_name_1': 'cliPec',
            'child_type_2': 'privateLinkServiceConnectionsSubTypes',
            'child_name_2': 'cliPecSubName',
            'last_child_num': 2
        }
        self.assertEqual(len(result.keys()), len(valid_dict_values.keys()))
        for key, value in valid_dict_values.items():
            self.assertEqual(result[key], value)

        invalid_proxy_resource_id = "invalidProxyResourceID"
        result = parse_proxy_resource_id(invalid_proxy_resource_id)
        self.assertIsNone(result)

    @mock.patch('webbrowser.open', autospec=True)
    @mock.patch('subprocess.Popen', autospec=True)
    def test_open_page_in_browser(self, subprocess_open_mock, webbrowser_open_mock):
        platform = sys.platform.lower()
        open_page_in_browser("http://foo")
        if is_wsl():
            subprocess_open_mock.assert_called_once_with(['powershell.exe', '-NoProfile',
                                                          '-Command', "Start-Process 'http://foo'"])
        elif platform == 'darwin':
            subprocess_open_mock.assert_called_once_with(['open', 'http://foo'])
        else:
            webbrowser_open_mock.assert_called_once_with('http://foo', 2)

    @mock.patch('shutil.which', autospec=True)
    @mock.patch('azure.cli.core.util._get_platform_info', autospec=True)
    @mock.patch('webbrowser.get', autospec=True)
    def test_can_launch_browser(self, webbrowser_get_mock, get_platform_mock, which_mock):
        import webbrowser

        # Windows is always fine
        get_platform_mock.return_value = ('windows', '10')
        assert can_launch_browser()

        # MacOS is always fine
        get_platform_mock.return_value = ('darwin', '10')
        assert can_launch_browser()

        # Real linux with browser
        get_platform_mock.return_value = ('linux', '4.15.0-1014-azure')
        browser_mock = mock.MagicMock()
        browser_mock.name = 'www-browser'
        webbrowser_get_mock.return_value = browser_mock
        assert can_launch_browser()

        # Real linux without browser
        get_platform_mock.return_value = ('linux', '4.15.0-1014-azure')
        webbrowser_get_mock.side_effect = webbrowser.Error
        assert not can_launch_browser()

        # WSL Linux with www-browser
        get_platform_mock.return_value = ('linux', '5.10.16.3-microsoft-standard-WSL2')
        browser_mock = mock.MagicMock()
        browser_mock.name = 'www-browser'
        webbrowser_get_mock.return_value = browser_mock
        assert can_launch_browser()

        # WSL Linux without www-browser, but with powershell.exe
        get_platform_mock.return_value = ('linux', '5.10.16.3-microsoft-standard-WSL2')
        webbrowser_get_mock.side_effect = webbrowser.Error
        which_mock.return_value = True
        assert can_launch_browser()

        # Docker container on WSL 2 can't launch browser
        get_platform_mock.return_value = ('linux', '5.10.16.3-microsoft-standard-WSL2')
        import webbrowser
        webbrowser_get_mock.side_effect = webbrowser.Error
        which_mock.return_value = False
        assert not can_launch_browser()

    def test_configured_default_setter(self):
        config = mock.MagicMock()
        config.use_local_config = None
        with ConfiguredDefaultSetter(config, True):
            self.assertEqual(config.use_local_config, True)
        self.assertIsNone(config.use_local_config)

        config.use_local_config = True
        with ConfiguredDefaultSetter(config, False):
            self.assertEqual(config.use_local_config, False)
        self.assertTrue(config.use_local_config)

    @mock.patch('azure.cli.core.__version__', '7.8.9')
    def test_get_az_user_agent(self):
        from azure.cli.core._environment import _ENV_AZ_INSTALLER
        with mock.patch.dict('os.environ', {_ENV_AZ_INSTALLER: 'PIP'}):
            actual = get_az_user_agent()
            self.assertEqual(actual, 'AZURECLI/7.8.9 (PIP)')

        actual = get_az_user_agent()
        self.assertEqual(actual, 'AZURECLI/7.8.9')

    @mock.patch.dict('os.environ')
    @mock.patch('azure.cli.core._profile.Profile.get_raw_token', autospec=True)
    @mock.patch('requests.Session.send', autospec=True)
    def test_send_raw_requests(self, send_mock, get_raw_token_mock):
        if 'AZURE_HTTP_USER_AGENT' in os.environ:
            del os.environ['AZURE_HTTP_USER_AGENT']  # Clear env var possibly added by DevOps

        return_val = mock.MagicMock()
        return_val.is_ok = True
        send_mock.return_value = return_val
        get_raw_token_mock.return_value = ("Bearer", "eyJ0eXAiOiJKV1", None), None, None

        cli_ctx = DummyCli()
        cli_ctx.data = {
            'command': 'rest',
            'safe_params': ['method', 'uri']
        }
        test_arm_active_directory_resource_id = 'https://management.core.windows.net/'
        test_arm_endpoint = 'https://management.azure.com/'
        subscription_id = '00000001-0000-0000-0000-000000000000'
        arm_resource_id = '/subscriptions/{}/resourcegroups/02?api-version=2019-07-01'.format(subscription_id)
        full_arm_rest_url = test_arm_endpoint.rstrip('/') + arm_resource_id
        test_body = '{"b1": "v1"}'

        expected_header = {
            'User-Agent': get_az_rest_user_agent(),
            'Accept': '*/*',
            'Connection': 'keep-alive',
            'Content-Type': 'application/json',
            'CommandName': 'rest',
            'ParameterSetName': 'method uri',
            'Content-Length': '12'
        }
        expected_header_with_auth = expected_header.copy()
        expected_header_with_auth['Authorization'] = 'Bearer eyJ0eXAiOiJKV1'

        # Test basic usage
        # Mock Put Blob https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob
        # Authenticate with service SAS https://learn.microsoft.com/en-us/rest/api/storageservices/create-service-sas
        sas_token = ['sv=2019-02-02', '{"srt": "s"}', "{'ss': 'bf'}"]
        send_raw_request(cli_ctx, 'PUT', 'https://myaccount.blob.core.windows.net/mycontainer/myblob?timeout=30',
                         uri_parameters=sas_token, body=test_body,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_not_called()
        request = send_mock.call_args[0][1]
        self.assertEqual(request.method, 'PUT')
        self.assertEqual(request.url, 'https://myaccount.blob.core.windows.net/mycontainer/myblob?timeout=30&sv=2019-02-02&srt=s&ss=bf')
        self.assertEqual(request.body, '{"b1": "v1"}')
        # Verify no Authorization header
        self._assert_headers(request.headers, expected_header)
        self.assertEqual(send_mock.call_args[1]["verify"], not should_disable_connection_verify())

        # Test Authorization header is skipped
        send_raw_request(cli_ctx, 'GET', full_arm_rest_url, body=test_body, skip_authorization_header=True,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_not_called()
        request = send_mock.call_args[0][1]
        self._assert_headers(request.headers, expected_header)

        # Test Authorization header is already provided
        send_raw_request(cli_ctx, 'GET', full_arm_rest_url,
                         body=test_body, headers={'Authorization=Basic ABCDE'},
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_not_called()
        request = send_mock.call_args[0][1]
        self._assert_headers(request.headers, {**expected_header, 'Authorization': 'Basic ABCDE'})

        # Test Authorization header is auto appended
        send_raw_request(cli_ctx, 'GET', full_arm_rest_url,
                         body=test_body,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id, subscription=subscription_id)
        request = send_mock.call_args[0][1]
        self._assert_headers(request.headers, expected_header_with_auth)

        # Test ARM Subscriptions - List
        # https://learn.microsoft.com/en-us/rest/api/resources/subscriptions/list
        # /subscriptions?api-version=2020-01-01
        send_raw_request(cli_ctx, 'GET', '/subscriptions?api-version=2020-01-01', body=test_body,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id)
        request = send_mock.call_args[0][1]
        self.assertEqual(request.url, test_arm_endpoint.rstrip('/') + '/subscriptions?api-version=2020-01-01')
        self._assert_headers(request.headers, expected_header_with_auth)

        # Test ARM Tenants - List
        # https://learn.microsoft.com/en-us/rest/api/resources/tenants/list
        # /tenants?api-version=2020-01-01
        send_raw_request(cli_ctx, 'GET', '/tenants?api-version=2020-01-01', body=test_body,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id)
        request = send_mock.call_args[0][1]
        self.assertEqual(request.url, test_arm_endpoint.rstrip('/') + '/tenants?api-version=2020-01-01')
        self._assert_headers(request.headers, expected_header_with_auth)

        # Test ARM resource ID
        # /subscriptions/00000001-0000-0000-0000-000000000000/resourcegroups/02?api-version=2019-07-01
        send_raw_request(cli_ctx, 'GET', arm_resource_id, body=test_body,
                         generated_client_request_id_name=None)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id, subscription=subscription_id)
        request = send_mock.call_args[0][1]
        self.assertEqual(request.url, full_arm_rest_url)
        self._assert_headers(request.headers, expected_header_with_auth)

        # Test full ARM URL
        # https://management.azure.com/subscriptions/00000001-0000-0000-0000-000000000000/resourcegroups/02?api-version=2019-07-01
        send_raw_request(cli_ctx, 'GET', full_arm_rest_url)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id, subscription=subscription_id)
        request = send_mock.call_args[0][1]
        self.assertEqual(request.url, full_arm_rest_url)

        # Test full ARM URL with port
        # https://management.azure.com:443/subscriptions/00000001-0000-0000-0000-000000000000/resourcegroups/02?api-version=2019-07-01
        test_arm_endpoint_with_port = 'https://management.azure.com:443/'
        full_arm_rest_url_with_port = test_arm_endpoint_with_port.rstrip('/') + arm_resource_id
        send_raw_request(cli_ctx, 'GET', full_arm_rest_url_with_port)

        get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id, subscription=subscription_id)
        request = send_mock.call_args[0][1]
        self.assertEqual(request.url, 'https://management.azure.com:443/subscriptions/00000001-0000-0000-0000-000000000000/resourcegroups/02?api-version=2019-07-01')

        # Test lookalike host is NOT mistaken for the trusted ARM endpoint via prefix matching.
        # The access token must NOT be attached to an attacker-controlled origin.
        for spoofed_host in ['https://management.azure.com.attacker', 'https://management.azure.com@attacker']:
            spoofed_url = spoofed_host + arm_resource_id
            get_raw_token_mock.reset_mock()
            send_raw_request(cli_ctx, 'GET', spoofed_url, generated_client_request_id_name=None)
            get_raw_token_mock.assert_not_called()
            request = send_mock.call_args[0][1]

            self.assertNotIn('Authorization', request.headers)

        # Test non-ARM APIs

        # Test AD Graph API https://graph.windows.net/
        url = 'https://graph.windows.net/00000002-0000-0000-0000-000000000000/applications/00000003-0000-0000-0000-000000000000?api-version=1.6'
        send_raw_request(cli_ctx, 'PATCH', url, body=test_body, generated_client_request_id_name=None)
        get_raw_token_mock.assert_called_with(mock.ANY, 'https://graph.windows.net/')
        request = send_mock.call_args[0][1]
        self.assertEqual(request.method, 'PATCH')
        self.assertEqual(request.url, url)

        # Test MS Graph API https://graph.microsoft.com/beta/appRoleAssignments/01
        url = 'https://graph.microsoft.com/beta/appRoleAssignments/01'
        send_raw_request(cli_ctx, 'PATCH', url, body=test_body, generated_client_request_id_name=None)
        get_raw_token_mock.assert_called_with(mock.ANY, 'https://graph.microsoft.com/')
        request = send_mock.call_args[0][1]
        self.assertEqual(request.method, 'PATCH')
        self.assertEqual(request.url, url)

        # Test custom case-insensitive User-Agent
        with mock.patch.dict('os.environ', {'AZURE_HTTP_USER_AGENT': "env-ua"}):
            send_raw_request(cli_ctx, 'GET', full_arm_rest_url, headers={'user-agent=ARG-UA'})

            get_raw_token_mock.assert_called_with(mock.ANY, test_arm_active_directory_resource_id, subscription=subscription_id)
            request = send_mock.call_args[0][1]
            self.assertEqual(request.headers['User-Agent'], get_az_rest_user_agent() + ' env-ua ARG-UA')

    def test_is_same_origin(self):
        endpoint = 'https://management.azure.com/'

        # Same origin
        self.assertTrue(is_same_origin('https://management.azure.com/subscriptions/01?api-version=2025-09-01', endpoint))
        self.assertTrue(is_same_origin('https://management.azure.com', endpoint))
        # Case-insensitive scheme and host
        self.assertTrue(is_same_origin('HTTPS://Management.Azure.Com/path', endpoint))
        # Default port is equivalent to an omitted port
        self.assertTrue(is_same_origin('https://management.azure.com:443/path', endpoint))
        self.assertTrue(is_same_origin('https://management.azure.com/path', 'https://management.azure.com:443/'))

        # Prefix-matching attack: lookalike host must NOT match
        self.assertFalse(is_same_origin('https://management.azure.com.attacker/path', endpoint))
        self.assertFalse(is_same_origin('https://management.azure.com.attacker', endpoint))
        # Userinfo trick: real host is the attacker
        self.assertFalse(is_same_origin('https://management.azure.com@attacker/path', endpoint))
        # Different scheme
        self.assertFalse(is_same_origin('http://management.azure.com/path', endpoint))
        # Different port
        self.assertFalse(is_same_origin('https://management.azure.com:8443/path', endpoint))
        # Different host
        self.assertFalse(is_same_origin('https://graph.microsoft.com/path', endpoint))

        # Malformed / non-URL inputs
        self.assertFalse(is_same_origin('not a url', endpoint))
        self.assertFalse(is_same_origin('', endpoint))
        self.assertFalse(is_same_origin('https://management.azure.com/path', 'not a url'))

    @mock.patch("psutil.Process")
    def test_get_parent_proc_name(self, mock_process_type):
        process = mock_process_type.return_value
        parent1 = process.parent.return_value
        parent2 = parent1.parent.return_value
        parent3 = parent2.parent.return_value

        # Windows, in a virtual env, launched by pwsh.exe
        process.name.return_value = "python.exe"
        parent1.name.return_value = "python.exe"
        parent2.name.return_value = "cmd.exe"
        parent3.name.return_value = "pwsh.exe"
        self.assertEqual(_get_parent_proc_name(), "pwsh.exe")

        # Windows, in a virtual env, launched by powershell.exe
        parent3.name.return_value = "powershell.exe"
        self.assertEqual(_get_parent_proc_name(), "powershell.exe")

        # Windows, launched by cmd.exe
        parent1.name.return_value = "cmd.exe"
        parent2.name.return_value = "explorer.exe"
        self.assertEqual(_get_parent_proc_name(), "cmd.exe")

        # Linux, launched by bash
        process.name.return_value = "python"
        parent1.name.return_value = "bash"
        parent2.name.return_value = "init"
        self.assertEqual(_get_parent_proc_name(), "bash")

        # Linux, launched by pwsh, launched by bash
        process.name.return_value = "python"
        parent1.name.return_value = "pwsh"
        parent2.name.return_value = "bash"
        self.assertEqual(_get_parent_proc_name(), "pwsh")

    def test_cli_run_cmd(self):
        cmd = ["echo", "abc"]
        if platform.system().lower() == "windows":
            cmd = ["cmd.exe", "/c"] + cmd
        output = run_cmd(cmd, capture_output=True)
        self.assertEqual(output.returncode, 0, "error when run cmd in shell")
        self.assertEqual(output.stdout.decode("utf8").strip(), "abc", "unexpected output when run cmd")

        output = run_cmd(cmd, capture_output=True, encoding="utf8")
        self.assertEqual(output.returncode, 0, "error when run cmd in shell")
        self.assertEqual(output.stdout.strip(), "abc", "unexpected output when run cmd")

    def test_run_cmd_arg_error(self):
        cmd = "echo abc"
        if platform.system().lower() == "windows":
            cmd = "cmd.exe /c " + cmd
        from azure.cli.core.azclierror import ArgumentUsageError
        with self.assertRaises(ArgumentUsageError):
            run_cmd(cmd, check=True)

    def test_run_az_cmd(self):
        cmd = ["az", "version"]
        output = run_az_cmd(cmd)
        self.assertEqual(output.exit_code, 0, "unexpected exit_code when run az version")
        self.assertEqual(output.error, None, "unexpected error when run az cmd")
        self.assertIsInstance(output.result, dict, "unexpected cmd execution result")
        self.assertIn("azure-cli-core", output.result, "unexpected cmd execution result")


class TestBase64ToHex(unittest.TestCase):

    def setUp(self):
        self.base64 = 'PvOJgaPq5R004GyT1tB0IW3XUyM='.encode('ascii')

    def test_b64_to_hex(self):
        self.assertEqual('3EF38981A3EAE51D34E06C93D6D074216DD75323', b64_to_hex(self.base64))

    def test_b64_to_hex_type(self):
        self.assertIsInstance(b64_to_hex(self.base64), str)


class TestGetProperty(unittest.TestCase):

    def test_getprop(self):
        from azure.cli.core.util import getprop
        with self.assertRaises(AttributeError):
            getprop(self, '__class__')
        with self.assertRaises(AttributeError):
            getprop(self, '__init__')
        with self.assertRaises(AttributeError):
            getprop(self, 'assertRaises')
        with self.assertRaises(AttributeError):
            getprop(self, '_diffThreshold')
        with self.assertRaises(AttributeError):
            getprop(self, 'new_props')
        self.assertEqual(getprop(self, 'maxDiff'), self.maxDiff)
        self.assertEqual(getprop(self, 'new_props', "new_props"), "new_props")


class TestHandleException(unittest.TestCase):

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_keyboardinterrupt(self, mock_logger_error):
        # create test KeyboardInterrupt Exception
        keyboard_interrupt_ex = KeyboardInterrupt("KeyboardInterrupt")

        # call handle_exception
        ex_result = handle_exception(keyboard_interrupt_ex)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_clierror(self, mock_logger_error):
        from knack.util import CLIError

        # create test CLIError Exception
        err_msg = "Error Message"
        cli_error = CLIError(err_msg)

        # call handle_exception
        ex_result = handle_exception(cli_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIn(err_msg, mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_httpresponseerror(self, mock_logger_error):
        from azure.core.exceptions import HttpResponseError

        # create test HttpResponseError Exception
        mock_http_response_error = HttpResponseError(response=mock.MagicMock(status_code="xxx",
                                                                             reason="There was a Http Response Error."))

        # call handle_exception
        ex_result = handle_exception(mock_http_response_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIn(mock_http_response_error.args[0], mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_httpoperationerror_typical_response_error(self, mock_logger_error):
        # create test HttpOperationError Exception
        err_msg = "Bad Request because of some incorrect param"
        err_code = "BadRequest"
        err = {"error": {"code": err_code, "message": err_msg}}
        response_text = json.dumps(err)
        mock_http_error = self._get_mock_HttpOperationError(response_text)

        # call handle_exception
        ex_result = handle_exception(mock_http_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIn(err_msg, mock_logger_error.call_args[0][0])
        self.assertIn(err_code, mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_httpoperationerror_error_key_has_string_value(self, mock_logger_error):
        # test error in response, but has str value.

        # create test HttpOperationError Exception
        err_msg = "BadRequest"
        err = {"error": err_msg}
        response_text = json.dumps(err)
        mock_http_error = self._get_mock_HttpOperationError(response_text)

        expected_message = "{}".format(err_msg)

        # call handle_exception
        ex_result = handle_exception(mock_http_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIn(expected_message, mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_httpoperationerror_no_error_key(self, mock_logger_error):
        # test error not in response

        # create test HttpOperationError Exception
        err_msg = "BadRequest"
        err = dict(foo=err_msg)
        response_text = json.dumps(err)
        mock_http_error = self._get_mock_HttpOperationError(response_text)

        # call handle_exception
        ex_result = handle_exception(mock_http_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIs(str(mock_http_error), mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @mock.patch('azure.cli.core.azclierror.logger.error', autospec=True)
    def test_handle_exception_httpoperationerror_no_response_text(self, mock_logger_error):
        # test no response text

        # create test HttpOperationError Exception
        response_text = ""

        mock_http_error = self._get_mock_HttpOperationError(response_text)

        # call handle_exception
        ex_result = handle_exception(mock_http_error)

        # test behavior
        self.assertTrue(mock_logger_error.called)
        self.assertIn(str(mock_http_error), mock_logger_error.call_args[0][0])
        self.assertEqual(ex_result, 1)

    @staticmethod
    def _get_mock_HttpOperationError(response_text):
        from msrest.exceptions import HttpOperationError
        from requests import Response

        mock_response = mock.MagicMock(spec=Response)
        mock_response.text = response_text
        mock_http_error = mock.MagicMock(spec=HttpOperationError)
        mock_http_error.response = mock_response

        return mock_http_error

    def test_roughly_parse_command(self):
        """Test roughly_parse_command function that extracts command parts and converts to lowercase"""
        # Basic command parsing
        self.assertEqual(roughly_parse_command(['az', 'vm', 'create']), 'az vm create')
        self.assertEqual(roughly_parse_command(['account', 'show']), 'account show')
        self.assertEqual(roughly_parse_command(['network', 'vnet', 'list']), 'network vnet list')
        
        # Test case conversion - should convert to lowercase
        self.assertEqual(roughly_parse_command(['az', 'VM', 'CREATE']), 'az vm create')
        self.assertEqual(roughly_parse_command(['Account', 'Show']), 'account show')
        
        # Test with flags - should stop at first flag and not include flag values
        self.assertEqual(roughly_parse_command(['az', 'vm', 'create', '--name', 'secretVM']), 'az vm create')
        self.assertEqual(roughly_parse_command(['az', 'storage', 'account', 'create', '--name', 'mystorageaccount']), 'az storage account create')
        self.assertEqual(roughly_parse_command(['az', 'keyvault', 'create', '--resource-group', 'myRG', '--name', 'myVault']), 'az keyvault create')
        
        # Test with short flags
        self.assertEqual(roughly_parse_command(['az', 'vm', 'list', '-g', 'myResourceGroup']), 'az vm list')
        self.assertEqual(roughly_parse_command(['az', 'group', 'create', '-n', 'myGroup', '-l', 'eastus']), 'az group create')
        
        # Edge cases
        self.assertEqual(roughly_parse_command([]), '')
        self.assertEqual(roughly_parse_command(['az']), 'az')
        self.assertEqual(roughly_parse_command(['--help']), '')  # Starts with flag
        self.assertEqual(roughly_parse_command(['-h']), '')  # Starts with short flag

if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from unittest import mock

from azure.cli.core.azclierror import ArgumentUsageError

from azure.cli.command_modules.appservice._validators import (
    _normalize_http_headers,
    _normalize_ip_address_list,
    _validate_ip_address_existence,
    _validate_service_tag_existence,
)


class _StubRule:
    """Minimal stand-in for azure.mgmt.web.models.IpSecurityRestriction."""

    def __init__(self, ip_address, headers=None):
        self.ip_address = ip_address
        self.headers = headers


class NormalizeHttpHeadersTest(unittest.TestCase):
    def test_none_and_empty_normalize_equally(self):
        self.assertEqual(_normalize_http_headers(None), {})
        self.assertEqual(_normalize_http_headers([]), {})
        self.assertEqual(_normalize_http_headers({}), {})

    def test_cli_form_lowercases_header_names(self):
        result = _normalize_http_headers(['X-Forwarded-For=10.0.0.1/32'])
        self.assertEqual(result, {'x-forwarded-for': frozenset({'10.0.0.1/32'})})

    def test_cli_form_repeated_names_accumulate(self):
        result = _normalize_http_headers([
            'x-forwarded-for=10.0.0.1/32',
            'x-forwarded-for=10.0.0.2/32',
        ])
        self.assertEqual(result, {'x-forwarded-for': frozenset({'10.0.0.1/32', '10.0.0.2/32'})})

    def test_sdk_form_value_order_insensitive(self):
        a = _normalize_http_headers({'x-forwarded-for': ['10.0.0.1/32', '10.0.0.2/32']})
        b = _normalize_http_headers({'x-forwarded-for': ['10.0.0.2/32', '10.0.0.1/32']})
        self.assertEqual(a, b)

    def test_cli_and_sdk_forms_compare_equal(self):
        cli = _normalize_http_headers([
            'x-forwarded-for=10.0.0.1/32',
            'x-forwarded-for=10.0.0.2/32',
        ])
        sdk = _normalize_http_headers({'x-forwarded-for': ['10.0.0.2/32', '10.0.0.1/32']})
        self.assertEqual(cli, sdk)

    def test_drops_empty_values(self):
        self.assertEqual(_normalize_http_headers(['x-forwarded-for=']), {})
        self.assertEqual(_normalize_http_headers({'x-forwarded-for': [None, '']}), {})

    def test_ignores_malformed_cli_entries_without_equals(self):
        self.assertEqual(_normalize_http_headers(['no-equals-here']), {})


class NormalizeIpAddressListTest(unittest.TestCase):
    def test_empty_inputs(self):
        self.assertEqual(_normalize_ip_address_list(None), frozenset())
        self.assertEqual(_normalize_ip_address_list(''), frozenset())

    def test_single_cidr(self):
        self.assertEqual(
            _normalize_ip_address_list('10.0.0.1/32'),
            frozenset({'10.0.0.1/32'}),
        )

    def test_order_independent(self):
        a = _normalize_ip_address_list('10.0.0.1/32,10.0.0.2/32')
        b = _normalize_ip_address_list('10.0.0.2/32,10.0.0.1/32')
        self.assertEqual(a, b)

    def test_strips_whitespace_around_entries(self):
        self.assertEqual(
            _normalize_ip_address_list('10.0.0.1/32, 10.0.0.2/32'),
            frozenset({'10.0.0.1/32', '10.0.0.2/32'}),
        )


class ValidateIpAddressExistenceTest(unittest.TestCase):
    def _make_namespace(self, ip_address, http_headers=None, scm_site=False):
        ns = mock.MagicMock()
        ns.resource_group_name = 'rg'
        ns.name = 'app'
        ns.slot = None
        ns.scm_site = scm_site
        ns.ip_address = ip_address
        ns.http_headers = http_headers
        return ns

    def _patch_configs(self, rules):
        configs = mock.MagicMock()
        configs.ip_security_restrictions = rules
        configs.scm_ip_security_restrictions = []
        return mock.patch(
            'azure.cli.command_modules.appservice._validators._generic_site_operation',
            return_value=configs,
        )

    def test_blocks_exact_duplicate_no_headers(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('36.12.195.236/32')
        with self._patch_configs([_StubRule('36.12.195.236/32')]):
            with self.assertRaises(ArgumentUsageError) as ctx:
                _validate_ip_address_existence(cmd, ns)
        msg = str(ctx.exception)
        self.assertIn('already exists', msg)
        self.assertNotIn('HTTP header filter', msg)
        self.assertIn('add a --http-header filter', msg)

    def test_allows_same_ip_with_different_xff(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            '36.12.195.236/32',
            http_headers=['x-forwarded-for=10.0.0.2/32'],
        )
        existing = _StubRule(
            '36.12.195.236/32',
            headers={'x-forwarded-for': ['10.0.0.1/32']},
        )
        with self._patch_configs([existing]):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Different XFF filter should be allowed')

    def test_allows_same_ip_with_headers_vs_no_headers(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            '36.12.195.236/32',
            http_headers=['x-forwarded-for=10.0.0.1/32'],
        )
        with self._patch_configs([_StubRule('36.12.195.236/32', headers=None)]):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Existing rule without headers should not block a header-filtered rule')

    def test_blocks_same_ip_and_identical_headers(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            '36.12.195.236/32',
            http_headers=['x-forwarded-for=10.0.0.1/32'],
        )
        existing = _StubRule(
            '36.12.195.236/32',
            headers={'x-forwarded-for': ['10.0.0.1/32']},
        )
        with self._patch_configs([existing]):
            with self.assertRaises(ArgumentUsageError) as ctx:
                _validate_ip_address_existence(cmd, ns)
        msg = str(ctx.exception)
        self.assertIn('HTTP header filter', msg)
        self.assertIn('vary the --http-header values', msg)

    def test_blocks_when_value_order_differs_for_same_header(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            '36.12.195.236/32',
            http_headers=[
                'x-forwarded-for=10.0.0.2/32',
                'x-forwarded-for=10.0.0.1/32',
            ],
        )
        existing = _StubRule(
            '36.12.195.236/32',
            headers={'x-forwarded-for': ['10.0.0.1/32', '10.0.0.2/32']},
        )
        with self._patch_configs([existing]):
            with self.assertRaises(ArgumentUsageError):
                _validate_ip_address_existence(cmd, ns)

    def test_allows_different_ip(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('36.12.195.236/32')
        with self._patch_configs([_StubRule('1.2.3.4/32')]):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Different IP should not be blocked')

    def test_handles_none_access_rules(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('10.0.0.1/32')
        with self._patch_configs(None):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('No existing rules should not raise')

    def test_multi_ip_blocks_when_order_differs_but_set_matches(self):
        """Comma-separated IP lists in a single rule are unordered for ARM purposes."""
        cmd = mock.MagicMock()
        ns = self._make_namespace('10.0.0.2/32,10.0.0.1/32')
        existing = _StubRule('10.0.0.1/32,10.0.0.2/32')
        with self._patch_configs([existing]):
            with self.assertRaises(ArgumentUsageError):
                _validate_ip_address_existence(cmd, ns)

    def test_multi_ip_allows_when_set_differs(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('10.0.0.1/32,10.0.0.3/32')
        existing = _StubRule('10.0.0.1/32,10.0.0.2/32')
        with self._patch_configs([existing]):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Different IP set should not be blocked')

    def test_scm_path_is_isolated_from_main(self):
        """scm_site=True must inspect scm_ip_security_restrictions only."""
        cmd = mock.MagicMock()
        ns = self._make_namespace('36.12.195.236/32', scm_site=True)
        configs = mock.MagicMock()
        configs.ip_security_restrictions = [_StubRule('36.12.195.236/32')]
        configs.scm_ip_security_restrictions = []
        with mock.patch(
            'azure.cli.command_modules.appservice._validators._generic_site_operation',
            return_value=configs,
        ):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Main-site duplicate must not affect SCM path')

    def test_namespace_without_http_headers_attr(self):
        """Real argparse Namespaces may omit http_headers when not declared."""
        from types import SimpleNamespace
        cmd = mock.MagicMock()
        ns = SimpleNamespace(
            resource_group_name='rg',
            name='app',
            slot=None,
            scm_site=False,
            ip_address='10.0.0.1/32',
        )
        with self._patch_configs([]):
            try:
                _validate_ip_address_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Missing http_headers attribute should be treated as no headers')


class ValidateServiceTagExistenceTest(unittest.TestCase):
    def _make_namespace(self, service_tag, http_headers=None, scm_site=False):
        ns = mock.MagicMock()
        ns.resource_group_name = 'rg'
        ns.name = 'app'
        ns.slot = None
        ns.scm_site = scm_site
        ns.service_tag = service_tag
        ns.http_headers = http_headers
        return ns

    def _patch_configs(self, rules):
        configs = mock.MagicMock()
        configs.ip_security_restrictions = rules
        configs.scm_ip_security_restrictions = []
        return mock.patch(
            'azure.cli.command_modules.appservice._validators._generic_site_operation',
            return_value=configs,
        )

    def test_blocks_exact_duplicate(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('AzureFrontDoor.Backend')
        with self._patch_configs([_StubRule('AzureFrontDoor.Backend')]):
            with self.assertRaises(ArgumentUsageError):
                _validate_service_tag_existence(cmd, ns)

    def test_allows_same_tag_with_different_fdid(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            'AzureFrontDoor.Backend',
            http_headers=['x-azure-fdid=22222222-2222-2222-2222-222222222222'],
        )
        existing = _StubRule(
            'AzureFrontDoor.Backend',
            headers={'x-azure-fdid': ['11111111-1111-1111-1111-111111111111']},
        )
        with self._patch_configs([existing]):
            try:
                _validate_service_tag_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('Different x-azure-fdid filter should be allowed')

    def test_blocks_when_tag_and_headers_match(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace(
            'AzureFrontDoor.Backend',
            http_headers=['x-azure-fdid=11111111-1111-1111-1111-111111111111'],
        )
        existing = _StubRule(
            'AzureFrontDoor.Backend',
            headers={'x-azure-fdid': ['11111111-1111-1111-1111-111111111111']},
        )
        with self._patch_configs([existing]):
            with self.assertRaises(ArgumentUsageError):
                _validate_service_tag_existence(cmd, ns)

    def test_handles_none_access_rules(self):
        cmd = mock.MagicMock()
        ns = self._make_namespace('AzureFrontDoor.Backend')
        with self._patch_configs(None):
            try:
                _validate_service_tag_existence(cmd, ns)
            except ArgumentUsageError:
                self.fail('No existing rules should not raise')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from datetime import datetime
from azure.cli.core.util import user_confirmation
from knack.log import get_logger
from ..utils.validators import validate_resource_group, validate_backup_name

logger = get_logger(__name__)

_BACKUP_NAME_PREFIX = "ondemandbackup"


def _generate_backup_name(client, resource_group_name, server_name):
    existing_backups = client.list_by_server(resource_group_name, server_name)
    existing_names = {backup.name for backup in existing_backups}

    on_demand_count = sum(1 for name in existing_names if name.startswith(_BACKUP_NAME_PREFIX))

    date_str = datetime.utcnow().strftime("%m%d%Y")
    backup_name = f"{_BACKUP_NAME_PREFIX}-{date_str}-{on_demand_count + 1}"

    if backup_name in existing_names:
        backup_name = f"{_BACKUP_NAME_PREFIX}-{date_str}-{on_demand_count + 2}"

    return backup_name


def backup_create_func(client, resource_group_name, server_name, backup_name=None):
    validate_resource_group(resource_group_name)

    if not backup_name:
        backup_name = _generate_backup_name(client, resource_group_name, server_name)
        logger.warning("No backup name provided. Using generated name: %s", backup_name)
    else:
        validate_backup_name(backup_name)

    return client.begin_create(
        resource_group_name,
        server_name,
        backup_name)


def backup_delete_func(client, resource_group_name, server_name, backup_name, yes=False):
    validate_resource_group(resource_group_name)

    if not yes:
        user_confirmation(
            "Are you sure you want to delete the backup '{0}' in server '{1}'".format(backup_name, server_name), yes=yes)

    return client.begin_delete(
        resource_group_name,
        server_name,
        backup_name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-many-lines

from knack.help_files import helps  # pylint: disable=unused-import



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long

from azure.cli.core.aaz import AAZStrArg, AAZListArg, register_command
from azure.cli.command_modules.monitor.aaz.latest.monitor.activity_log.alert._update \
    import Update as _ActivityLogAlertUpdate
from azure.cli.command_modules.monitor.operations.activity_log_alerts import normalize_names


@register_command("monitor activity-log alert action-group remove")
class ActivityLogAlertActionGroupRemove(_ActivityLogAlertUpdate):
    """Remove action groups from this activity log alert rule.
    """

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.action_groups._registered = False
        args_schema.all_of._registered = False
        args_schema.description._registered = False
        args_schema.enabled._registered = False
        args_schema.scopes._registered = False
        args_schema.tags._registered = False
        args_schema.action_group_ids = AAZListArg(
            options=["--action-group", "-a"],
            required=True,
            help="The names or the resource ids of the action groups to be added.",
        )
        args_schema.action_group_ids.Element = AAZStrArg()
        return args_schema

    def pre_instance_update(self, instance):
        args = self.ctx.args
        action_group_ids = args.action_group_ids.to_serialized_data()
        if len(action_group_ids) == 1 and action_group_ids[0] == '*':
            instance.properties.actions.actionGroups = []
        else:
            # normalize the action group ids
            rids = normalize_names(self.cli_ctx, args.action_group_ids.to_serialized_data(), args.resource_group,
                                   'microsoft.insights', 'actionGroups')
            action_groups = []
            for item in instance.properties.actions.actionGroups:
                ac_id = item.actionGroupId.to_serialized_data()
                found = False
                for rid in rids:
                    if ac_id.lower() == rid.lower():  # service returned action group id can be uppercase
                        found = True
                        break
                if not found:
                    action_groups.append(item)
            instance.properties.actions.actionGroups = action_groups



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long

from azure.cli.core.aaz import has_value, AAZStrArg, AAZListArg, AAZBoolArg, \
    AAZResourceIdArg, AAZResourceIdArgFormat
from azure.cli.command_modules.monitor.actions import AAZCustomListArg
from azure.cli.command_modules.monitor.aaz.latest.monitor.activity_log.alert._create \
    import Create as _ActivityLogAlertCreate
from azure.cli.command_modules.monitor.operations.activity_log_alerts import (
    _get_alert_settings_for_alert,
    process_condition_parameter_for_alert,
    process_webhook_properties,
)
from azure.cli.core.azclierror import ValidationError


class ActivityLogAlertCreate(_ActivityLogAlertCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.enabled._registered = False
        args_schema.location._registered = False
        args_schema.action_groups._registered = False
        args_schema.scopes._registered = False
        args_schema.scope_ui = AAZListArg(
            options=["--scope", "-s"],
            help="A list of strings that will be used as prefixes." + '''
        The alert rule will only apply to activity logs with resourceIDs that fall under one of
        these prefixes. If not provided, the subscriptionId will be used.
        ''',
        )
        args_schema.scope_ui.Element = AAZStrArg()

        args_schema.disable = AAZBoolArg(
            options=["--disable"],
            help="Disable the activity log alert rule after it is created.",
            default=False,
        )
        args_schema.condition = AAZCustomListArg(
            options=["--condition", "-c"],
            help="The condition that will cause the alert rule to activate. "
                 "The format is FIELD=VALUE[ and FIELD=VALUE...]" + '''
        The possible values for the field are 'resourceId', 'category', 'caller',
        'level', 'operationName', 'resourceGroup', 'resourceProvider', 'status',
        'subStatus', 'resourceType', or anything beginning with 'properties'.
        '''
        )
        args_schema.condition.Element = AAZStrArg()

        args_schema.action_group_ids = AAZListArg(
            options=["--action-group", "-a"],
            help="Add an action group. Accepts space-separated action group identifiers. "
                 "The identifier can be the action group's name or its resource ID.",
        )
        args_schema.action_group_ids.Element = AAZResourceIdArg(
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{subscription}/resourceGroups/{resource_group}/providers/microsoft.insights/actionGroups/{}",
            )
        )

        args_schema.webhook_properties_list = AAZCustomListArg(
            options=['--webhook-properties', '-w'],
            help="Space-separated webhook properties in 'key[=value]' format. "
                 "These properties are associated with the action groups added in this command." + '''
        For any webhook receiver in these action group, this data is appended to the webhook
        payload. To attach different webhook properties to different action groups, add the
        action groups in separate update-action commands.
        '''
        )
        args_schema.webhook_properties_list.Element = AAZStrArg()

        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.location = "Global"
        process_condition_parameter_for_alert(args)
        webhook_properties = process_webhook_properties(args)
        if not has_value(args.scope_ui):
            from azure.mgmt.core.tools import resource_id
            from azure.cli.core.commands.client_factory import get_subscription_id
            # args.scopes = [resource_id(subscription=get_subscription_id(self.cli_ctx),
            #                            resource_group=args.resource_group)]
            # service check
            args.scopes = [resource_id(subscription=get_subscription_id(self.cli_ctx))]
        else:
            args.scopes = args.scope_ui.to_serialized_data()
        if _get_alert_settings_for_alert(self, args.resource_group, args.activity_log_alert_name,
                                         throw_if_missing=False):
            raise ValidationError(
                'The activity log alert {} already exists in resource group {}.'.format(args.activity_log_alert_name,
                                                                                        args.resource_group))
        if not has_value(args.all_of):
            args.all_of.append({
                "field": "category",
                "equals": "ServiceHealth",
            })
        else:
            current_all_of = args.all_of.to_serialized_data()
            category_found = False
            for item in current_all_of:
                if item.get("field", None) == "category":
                    category_found = True
                    break
            if not category_found:
                args.all_of.append({
                    "field": "category",
                    "equals": "ServiceHealth",
                })
        # Add action groups
        action_group_rids = set()
        if has_value(args.action_group_ids):
            action_group_rids = set(args.action_group_ids.to_serialized_data())
        args.action_groups = []
        for i in action_group_rids:
            args.action_groups.append({
                "action_group_id": i,
                "webhook_properties": webhook_properties
            })
        if has_value(args.disable):
            args.enabled = not args.disable



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import parse_proxy_resource_id
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (
    JMESPathCheck,
    ResourceGroupPreparer,
    ScenarioTest)
from .constants import DEFAULT_LOCATION
from .server_preparer import ServerPreparer


class FlexibleServerPrivateEndpointsMgmtScenarioTest(ScenarioTest):

    postgres_location = DEFAULT_LOCATION

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    @ServerPreparer(location=postgres_location)
    def test_postgres_flexible_server_private_endpoint_mgmt(self, resource_group, server):
        self._test_private_endpoint_connection(resource_group, server)
        self._test_private_link_resource(resource_group, server, 'postgresqlServer')

    def _test_private_endpoint_connection(self, resource_group, server_name):
        loc = self.postgres_location
        vnet = self.create_random_name('cli-vnet-', 24)
        subnet = self.create_random_name('cli-subnet-', 24)
        pe_name_auto = self.create_random_name('cli-pe-', 24)
        pe_name_manual_approve = self.create_random_name('cli-pe-', 24)
        pe_name_manual_reject = self.create_random_name('cli-pe-', 24)
        pe_connection_name_auto = self.create_random_name('cli-pec-', 24)
        pe_connection_name_manual_approve = self.create_random_name('cli-pec-', 24)
        pe_connection_name_manual_reject = self.create_random_name('cli-pec-', 24)

        result = self.cmd('postgres flexible-server show -n {} -g {}'.format(server_name, resource_group),
                               checks=[JMESPathCheck('resourceGroup', resource_group),
                                       JMESPathCheck('name', server_name)]).get_output_in_json()
        self.assertEqual(''.join(result['location'].lower().split()), self.postgres_location)

        # Prepare network and disable network policies
        self.cmd('network vnet create -n {} -g {} -l {} --subnet-name {}'
                 .format(vnet, resource_group, loc, subnet),
                 checks=self.check('length(newVNet.subnets)', 1))
        self.cmd('network vnet subnet update -n {} --vnet-name {} -g {} '
                 '--private-endpoint-network-policies Disabled'
                 .format(subnet, vnet, resource_group),
                 checks=self.check('privateEndpointNetworkPolicies', 'Disabled'))

        # Get Server Id and Group Id
        result = self.cmd('postgres flexible-server show -g {} -n {}'
                          .format(resource_group, server_name)).get_output_in_json()
        server_id = result['id']
        group_id = 'postgresqlServer'

        approval_description = 'You are approved!'
        rejection_description = 'You are rejected!'

        # Testing Auto-Approval workflow
        # Create a private endpoint connection
        private_endpoint = self.cmd('network private-endpoint create -g {} -n {} --vnet-name {} --subnet {} -l {} '
                                    '--connection-name {} --private-connection-resource-id {} '
                                    '--group-id {}'
                                    .format(resource_group, pe_name_auto, vnet, subnet, loc, pe_connection_name_auto,
                                            server_id, group_id)).get_output_in_json()
        self.assertEqual(private_endpoint['name'], pe_name_auto)
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['name'], pe_connection_name_auto)
        self.assertEqual(
            private_endpoint['privateLinkServiceConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Approved')
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['provisioningState'], 'Succeeded')
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['groupIds'][0], group_id)

        # Get Private Endpoint Connection Name and Id
        result = self.cmd('postgres flexible-server show -g {} -n {}'
                          .format(resource_group, server_name)).get_output_in_json()
        self.assertEqual(len(result['privateEndpointConnections']), 1)
        self.assertEqual(
            result['privateEndpointConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Approved')
        server_pec_id = result['privateEndpointConnections'][0]['id']
        result = parse_proxy_resource_id(server_pec_id)
        server_pec_name = result['child_name_1']

        self.cmd('postgres flexible-server private-endpoint-connection show --server-name {} -g {} --name {}'
                 .format(server_name, resource_group, server_pec_name),
                 checks=[
                     self.check('id', server_pec_id),
                     self.check('privateLinkServiceConnectionState.status', 'Approved')
                 ])
        
        self.cmd('postgres flexible-server private-endpoint-connection approve --server-name {} -g {} --name {} --description "{}"'
                     .format(server_name, resource_group, server_pec_name, approval_description))

        self.cmd('postgres flexible-server private-endpoint-connection reject --server-name {} -g {} --name {} --description "{}"'
                     .format(server_name, resource_group, server_pec_name, rejection_description))

        self.cmd('postgres flexible-server private-endpoint-connection delete --server-name {} -g {} --id {}'
                 .format(server_name, resource_group, server_pec_id))

        # Testing Manual-Approval workflow [Approval]
        # Create a private endpoint connection
        private_endpoint = self.cmd('network private-endpoint create -g {} -n {} --vnet-name {} --subnet {} -l {} '
                                    '--connection-name {} --private-connection-resource-id {} '
                                    '--group-id {} --manual-request'
                                    .format(resource_group, pe_name_manual_approve, vnet, subnet, loc,
                                            pe_connection_name_manual_approve, server_id,
                                            group_id)).get_output_in_json()
        self.assertEqual(private_endpoint['name'], pe_name_manual_approve)
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['name'],
                         pe_connection_name_manual_approve)
        self.assertEqual(
            private_endpoint['manualPrivateLinkServiceConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Pending')
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['provisioningState'], 'Succeeded')
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['groupIds'][0], group_id)

        # Get Private Endpoint Connection Name and Id
        result = self.cmd('postgres flexible-server show -g {} -n {}'
                          .format(resource_group, server_name)).get_output_in_json()
        self.assertEqual(len(result['privateEndpointConnections']), 1)
        self.assertEqual(
            result['privateEndpointConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Pending')
        server_pec_id = result['privateEndpointConnections'][0]['id']
        result = parse_proxy_resource_id(server_pec_id)
        server_pec_name = result['child_name_1']

        self.cmd('postgres flexible-server private-endpoint-connection show --server-name {} -g {} --name {}'
                 .format(server_name, resource_group, server_pec_name),
                 checks=[
                     self.check('id', server_pec_id),
                     self.check('privateLinkServiceConnectionState.status', 'Pending'),
                     self.check('provisioningState', 'Succeeded')
                 ])

        self.cmd('postgres flexible-server private-endpoint-connection approve --server-name {} -g {} --name {} --description "{}"'
                 .format(server_name, resource_group, server_pec_name, approval_description),
                 checks=[
                     self.check('privateLinkServiceConnectionState.status', 'Approved'),
                     self.check('privateLinkServiceConnectionState.description', approval_description),
                     self.check('provisioningState', 'Succeeded')
                 ])

        self.cmd('postgres flexible-server private-endpoint-connection reject --server-name {} -g {} --name {} --description "{}"'
                     .format(server_name, resource_group, server_pec_name, rejection_description))

        self.cmd('postgres flexible-server private-endpoint-connection delete --server-name {} -g {}  --id {}'
                 .format(server_name, resource_group, server_pec_id))

        # Testing Manual-Approval workflow [Rejection]
        # Create a private endpoint connection
        private_endpoint = self.cmd('network private-endpoint create -g {} -n {} --vnet-name {} --subnet {} -l {} '
                                    '--connection-name {} --private-connection-resource-id {} '
                                    '--group-id {} --manual-request true'
                                    .format(resource_group, pe_name_manual_reject, vnet, subnet, loc,
                                            pe_connection_name_manual_reject, server_id, group_id)).get_output_in_json()
        self.assertEqual(private_endpoint['name'], pe_name_manual_reject)
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['name'],
                         pe_connection_name_manual_reject)
        self.assertEqual(
            private_endpoint['manualPrivateLinkServiceConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Pending')
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['provisioningState'], 'Succeeded')
        self.assertEqual(private_endpoint['manualPrivateLinkServiceConnections'][0]['groupIds'][0], group_id)

        # Get Private Endpoint Connection Name and Id
        result = self.cmd('postgres flexible-server show -g {} -n {}'
                          .format(resource_group, server_name)).get_output_in_json()
        self.assertEqual(len(result['privateEndpointConnections']), 1)
        self.assertEqual(
            result['privateEndpointConnections'][0]['privateLinkServiceConnectionState']['status'],
            'Pending')
        server_pec_id = result['privateEndpointConnections'][0]['id']
        result = parse_proxy_resource_id(server_pec_id)
        server_pec_name = result['child_name_1']

        self.cmd('postgres flexible-server private-endpoint-connection list -g {} --server-name {}'.format(resource_group, server_name),
                 checks=[JMESPathCheck('type(@)', 'array'),
                         JMESPathCheck('length(@)', 1)])

        self.cmd('postgres flexible-server private-endpoint-connection show --server-name {} -g {} --name {}'
                 .format(server_name, resource_group, server_pec_name),
                 checks=[
                     self.check('id', server_pec_id),
                     self.check('privateLinkServiceConnectionState.status', 'Pending'),
                     self.check('provisioningState', 'Succeeded')
                 ])

        self.cmd('postgres flexible-server private-endpoint-connection reject --server-name {} -g {} --name {} --description "{}"'
                 .format(server_name, resource_group, server_pec_name, rejection_description),
                 checks=[
                     self.check('privateLinkServiceConnectionState.status', 'Rejected'),
                     self.check('privateLinkServiceConnectionState.description', rejection_description),
                     self.check('provisioningState', 'Succeeded')
                 ])

        self.cmd('postgres flexible-server private-endpoint-connection approve --server-name {} -g {} --name {} --description "{}"'
                     .format(server_name, resource_group, server_pec_name, approval_description), expect_failure=True)

        self.cmd('postgres flexible-server private-endpoint-connection delete --server-name {} -g {}  --id {}'
                 .format(server_name, resource_group, server_pec_id))
        result = self.cmd('postgres flexible-server show -g {} -n {}'
                          .format(resource_group, server_name)).get_output_in_json()
        self.assertEqual(len(result['privateEndpointConnections']), 0)

    def _test_private_link_resource(self, resource_group, server, group_id):
        result = self.cmd('postgres flexible-server private-link-resource list -g {} -s {}'
                          .format(resource_group, server)).get_output_in_json()
        self.assertEqual(result[0]['groupId'], group_id)

        result = self.cmd('postgres flexible-server private-link-resource show -g {} -s {}'
                          .format(resource_group, server)).get_output_in_json()
        self.assertEqual(result['groupId'], group_id)


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.mgmt.core.tools import parse_resource_id

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, live_only)

from .utils import prepare_containerapp_env_for_app_e2e_tests

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppIngressStickySessionsTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    @live_only()  # encounters 'CannotOverwriteExistingCassetteException' only when run from recording (passes when run live)
    def test_containerapp_ingress_sticky_sessions_e2e(self, resource_group):

        app = self.create_random_name(prefix='app2', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])
        self.cmd("az containerapp create -g {} --target-port 80 --ingress external --image mcr.microsoft.com/k8se/quickstart:latest --environment {} -n {} ".format(resource_group, env_id, app))
        self.cmd("az containerapp ingress sticky-sessions set -n {} -g {} --affinity sticky".format(app, resource_group), expect_failure=False, checks=[
            JMESPathCheck('stickySessions.affinity', "sticky")
        ])
        self.cmd("az containerapp ingress sticky-sessions show -n {} -g {}".format(app, resource_group), expect_failure=False, checks=[
            JMESPathCheck('stickySessions.affinity', "sticky")
        ])
        self.cmd('containerapp show -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('properties.configuration.ingress.stickySessions.affinity', "sticky"),        
        ])


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import re
from random import randint
from unittest import mock


from knack.log import get_logger
from azure.cli.command_modules.aro.tests.latest.custom_preparers import AROClusterServicePrincipalPreparer
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from azure.cli.testsdk.checkers import StringContainCheck
from azure.cli.testsdk.scenario_tests import AllowLargeResponse

logger = get_logger(__name__)


class AroScenarioTests(ScenarioTest):
    def test_aro_get_versions(self):
        """Test aro get-versions command returns valid version format."""

        # Test get-versions - this is a lightweight test that doesn't create resources
        versions_output = self.cmd('aro get-versions --location eastus').get_output_in_json()

        # Validate we got a list
        self.assertTrue(isinstance(versions_output, list), "get-versions should return a list")
        self.assertGreater(len(versions_output), 0, "get-versions should return at least one version")

        # Validate each version matches semantic versioning format (x.y.z)
        version_pattern = re.compile(r'^\d+\.\d+\.\d+$')
        for version in versions_output:
            self.assertTrue(isinstance(version, str), f"Version {version} should be a string")
            self.assertTrue(version_pattern.match(version),
                            f"Version {version} should match x.y.z format")

    @AllowLargeResponse()
    @ResourceGroupPreparer(random_name_length=28, name_prefix='cli_test_aro', location='eastus')
    @AROClusterServicePrincipalPreparer(name_prefix='cli_test_aro')
    def test_aro_public_cluster(self, resource_group):
        from azure.mgmt.core.tools import resource_id

        subscription = self.get_subscription_id()

        master_subnet = self.create_random_name('dev_master', 14)
        worker_subnet = self.create_random_name('dev_worker', 14)
        name = self.create_random_name('aro', 14)

        temp_kubeconfig_path = self.create_random_name('kubeconfig', 24) + '.tmp'

        self.kwargs.update({
            'name': name,
            'resource_group': resource_group,
            'subscription': subscription,
            'master_subnet': master_subnet,
            'worker_subnet': worker_subnet,
            'master_ip_range': '10.{}.{}.0/24'.format(randint(0, 127), randint(0, 255)),
            'worker_ip_range': '10.{}.{}.0/24'.format(randint(0, 127), randint(0, 255)),
            'master_subnet_resource': resource_id(subscription=subscription, resource_group=resource_group, namespace='Microsoft.Network', type='virtualNetworks', child_type_1='subnets', name='dev-vnet', child_name_1=master_subnet),
            'worker_subnet_resource': resource_id(subscription=subscription, resource_group=resource_group, namespace='Microsoft.Network', type='virtualNetworks', child_type_1='subnets', name='dev-vnet', child_name_1=worker_subnet),
            'temp_kubeconfig_path': temp_kubeconfig_path,
        })

        self.cmd('network vnet create -g {rg} -n dev-vnet --address-prefixes 10.0.0.0/9')
        self.cmd('network vnet subnet create -g {rg} --vnet-name dev-vnet -n {master_subnet} --address-prefixes {master_ip_range} --service-endpoints Microsoft.ContainerRegistry --default-outbound false')
        self.cmd('network vnet subnet create -g {rg} --vnet-name dev-vnet -n {worker_subnet} --address-prefixes {worker_ip_range} --service-endpoints Microsoft.ContainerRegistry --default-outbound false')
        self.cmd('network vnet subnet update -g {rg} --vnet-name dev-vnet -n {master_subnet} --private-link-service-network-policies Disabled')

        # aro validate
        with mock.patch('azure.cli.command_modules.aro._rbac._gen_uuid', side_effect=self.create_guid):
            self.cmd('aro validate -g {rg} -n {name} --client-id {aro_csp} --client-secret {aro_csp_pass} --master-subnet {master_subnet_resource} --worker-subnet {worker_subnet_resource} --subscription {subscription}')

        # aro create
        with mock.patch('azure.cli.command_modules.aro._rbac._gen_uuid', side_effect=self.create_guid):
            self.cmd('aro create -g {rg} -n {name} --client-id {aro_csp} --client-secret {aro_csp_pass} --master-subnet {master_subnet_resource} --worker-subnet {worker_subnet_resource} --subscription {subscription} --tags test=create', checks=[
                self.check('tags.test', 'create'),
                self.check('name', '{name}'),
                self.check('masterProfile.subnetId', '{master_subnet_resource}'),
                self.check('workerProfiles[0].subnetId', '{worker_subnet_resource}'),
                self.check('provisioningState', 'Succeeded')
            ])

        # aro list-credentials
        self.cmd('aro list-credentials -g {rg} -n {name} --subscription {subscription}', checks=[self.check('kubeadminUsername', 'kubeadmin')])

        # aro get-admin-kubeconfig
        try:
            self.cmd('aro get-admin-kubeconfig -g {rg} -n {name} --subscription {subscription} -f {temp_kubeconfig_path}')
            self.assertGreater(os.path.getsize(temp_kubeconfig_path), 0)
        finally:
            os.remove(temp_kubeconfig_path)

        # aro show
        self.cmd('aro show -g {rg} -n {name} --subscription {subscription} --output table', checks=[
            StringContainCheck(name),
            StringContainCheck(resource_group),
            StringContainCheck('eastus'),
            StringContainCheck('Succeeded'),
        ])

        # aro list
        self.cmd('aro list -g {rg} --subscription {subscription}', checks=[
            self.check('[0].name', '{name}'),
            self.check('[0].provisioningState', 'Succeeded'),
            self.check_pattern('[0].id', '.*{name}')
        ])

        # aro update
        self.cmd('aro update -g {rg} -n {name} --subscription {subscription}', expect_failure=False)

        # aro delete
        self.cmd('aro delete -y -g {rg} -n {name} --subscription {subscription}', expect_failure=False)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import tempfile
import zipfile
import tarfile
import shutil
import unittest

from azure.cli.core.extension import DevExtension
try:
    from azure.cli.core.extension.tests.latest import ExtensionTypeTestMixin, get_test_data_file
except ImportError:
    from . import ExtensionTypeTestMixin, get_test_data_file


class TestWheelTypeExtensionMetadata(ExtensionTypeTestMixin):

    def test_reading_wheel_type_extension_from_develop_mode(self):
        """
        Test develop type extension.
        For scenario that user are developing extension via azdev
        """

        source_code_packaged = get_test_data_file('hello-0.1.0.tar.gz')

        with tarfile.open(source_code_packaged, 'r:gz') as tar:
            tar.extractall(self.ext_dir)

        ext_name, ext_version = 'hello', '0.1.0'

        ext_extension = DevExtension(ext_name, os.path.join(self.ext_dir, ext_name + '-' + ext_version))
        metadata = ext_extension.get_metadata()  # able to read metadata from source code

        # assert Python metadata
        self.assertEqual(metadata['name'], ext_name)
        self.assertEqual(metadata['version'], ext_version)
        self.assertEqual(metadata['author'], 'Microsoft Corporation')
        self.assertNotIn('metadata.json', os.listdir(os.path.join(self.ext_dir, ext_name + '-' + ext_version)))

        # assert Azure CLI extended metadata
        self.assertTrue(metadata['azext.isPreview'])
        self.assertTrue(metadata['azext.isExperimental'])
        self.assertEqual(metadata['azext.minCliCoreVersion'], '2.0.67')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import functools
import unittest
from .const import ENV_LIVE_TEST
from .utilities import trim_kwargs_from_test_function


def live_only():
    return unittest.skipUnless(
        os.environ.get(ENV_LIVE_TEST, False),
        'This is a live only test. A live test will bypass all vcrpy components.')


def record_only():
    return unittest.skipUnless(
        not os.environ.get(ENV_LIVE_TEST, False),
        'This test is excluded from being run live. To force a recording, please remove the recording file.')


class AllowLargeResponse:  # pylint: disable=too-few-public-methods

    def __init__(self, size_kb=1024):
        self.size_kb = size_kb

    def __call__(self, fn):
        def _preparer_wrapper(test_class_instance, **kwargs):
            from .recording_processors import LargeResponseBodyProcessor
            large_resp_body = next((r for r in test_class_instance.recording_processors
                                    if isinstance(r, LargeResponseBodyProcessor)), None)
            if large_resp_body:
                large_resp_body._max_response_body = self.size_kb  # pylint: disable=protected-access

            trim_kwargs_from_test_function(fn, kwargs)

            fn(test_class_instance, **kwargs)

        setattr(_preparer_wrapper, '__is_preparer', True)
        functools.update_wrapper(_preparer_wrapper, fn)
        return _preparer_wrapper



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import time

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, live_only
from knack.testsdk import JMESPathCheck

LOCATION_FOR_LIVE_TESTS = 'northeurope'


def _data_file(filename):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename).replace('\\', '\\\\')


@live_only()
class BatchAILiveScenariosTests(ScenarioTest):
    @ResourceGroupPreparer(location=LOCATION_FOR_LIVE_TESTS)
    def test_cluster_job_node_exec(self, resource_group):
        self.cmd('batchai workspace create -g {0} -n workspace'.format(resource_group))
        self.cmd('batchai experiment create -g {0} -w workspace -n experiment'.format(resource_group))

        # Create a cluster to test the ssh connection
        self.cmd('batchai cluster create -g {0} -w workspace -n simple -s Standard_D1 -t 1 '
                 '-u alex --generate-ssh-keys'.format(resource_group))

        # Schedule a job to know when the cluster is ready
        self.cmd('batchai job create -g {0} -w workspace -e experiment -n nop -c simple -f {1}'.format(
            resource_group, _data_file('nop_job.json')))
        self.cmd('batchai job wait -g {0} -w workspace -e experiment -n nop'.format(resource_group))

        # Submit a job which waits on a port 9000
        self.cmd('batchai job create -g {0} -w workspace -e experiment -n waiter1 -c simple -f {1}'.format(
            resource_group, _data_file('job_waiting_on_port.json')))

        # Give the service some time to actually start the job
        time.sleep(30)

        # The job should be in running state
        self.cmd('batchai job show -g {0} -w workspace -e experiment -n waiter1'.format(
            resource_group), checks=[JMESPathCheck('executionState', 'running')])

        # Write to port 9000 on the node
        self.cmd('batchai cluster node exec -g {0} -w workspace -c simple --exec "echo hi | nc localhost 9000"'.format(
            resource_group))

        # The job must success soon
        self.cmd('batchai job wait -g {0} -w workspace -e experiment -n waiter1'.format(resource_group))

        # Check the same but using 'job node exec'
        self.cmd('batchai job create -g {0} -w workspace -e experiment -n waiter2 -c simple -f {1}'.format(
            resource_group, _data_file('job_waiting_on_port.json')))
        time.sleep(30)
        self.cmd('batchai job node exec -g {0} -w workspace -e experiment -j waiter2 '
                 '--exec "echo hi | nc localhost 9000"'.format(resource_group))
        self.cmd('batchai job wait -g {0} -w workspace -e experiment -n waiter2'.format(resource_group))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class AmsAccountTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_create_show(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'centralus',
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location} --mi-system-assigned --default-action Allow', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Central US'),
            self.check('identity.type', 'SystemAssigned')
        ])

        self.cmd('az ams account update -n {amsname} -g {rg} --tags key=value', checks=[
            self.check('tags.key', 'value'),
        ])

        list = self.cmd('az ams account list -g {}'.format(resource_group)).get_output_in_json()
        assert len(list) > 0

        self.cmd('az ams account show -n {amsname} -g {rg}', checks=[
            self.check('name', '{amsname}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('az ams account delete -n {amsname} -g {rg}')

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_sync_storage_keys(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'southeastasia',
        })

        account = self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location} --mi-system-assigned --default-action Allow').get_output_in_json()

        self.kwargs.update({
            'storageId': account['storageAccounts'][0]['id']
        })

        self.cmd('az ams account storage sync-storage-keys -g {rg} -a {amsname} --storage-account-id "{storageId}"')

        self.cmd('az ams account delete -n {amsname} -g {rg}')

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    @StorageAccountPreparer(parameter_name='storage_account_for_update')
    def test_ams_storage_add_remove(self, resource_group, storage_account_for_create, storage_account_for_update):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'eastasia'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location} --mi-system-assigned --default-action Allow', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'East Asia')
        ])

        self.kwargs.update({
            'storageAccount': storage_account_for_update
        })

        self.cmd('az ams account storage add -a {amsname} -g {rg} -n {storageAccount} --system-assigned', checks=[
            self.check('name', '{amsname}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('az ams account storage remove -a {amsname} -g {rg} -n {storageAccount}', checks=[
            self.check('name', '{amsname}'),
            self.check('resourceGroup', '{rg}')
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_check_name(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        amsname2 = 'unnombrequenoexiste'
        amsname3 = 'dgoifdgoisdfapodsgmpfdofmdspfoamdsfpodsamfpds%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'eastus',
            'amsname2': amsname2,
            'amsname3': amsname3
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location} --mi-system-assigned --default-action Allow')

        self.cmd('az ams account check-name --location {location} -n {amsname}', checks=[
            self.check('@', 'Already in use by another account. Please try again with a name that is not likely to be in use.')
        ])

        self.cmd('az ams account check-name --location {location} -n {amsname2}', checks=[
            self.check('@', 'Name available.')
        ])

        self.cmd('az ams account check-name --location {location} -n {amsname3}', checks=[
            self.check('@', 'The Media Account account name should be between 3 and 24 characters and may contain only lowercase letters and numbers.')
        ])

        self.cmd('az ams account delete -n {amsname} -g {rg}')
